@@ -34,7 +34,7 @@ static AtkStateSet*          gail_button_ref_state_set    (AtkObject       *obj)
 static void                  gail_button_notify_label_gtk (GObject         *obj,
                                                            GParamSpec      *pspec,
                                                            gpointer        data);
-static gint                  gail_button_label_map_gtk    (GtkWidget       *widget,
+static void                  gail_button_label_map_gtk    (GtkWidget       *widget,
                                                            gpointer        data);
 
 static void                  gail_button_real_initialize  (AtkObject       *obj,
@@ -333,7 +333,7 @@ gail_button_real_initialize (AtkObject *obj,
 
 }
 
-static gint
+static void
 gail_button_label_map_gtk (GtkWidget *widget,
                            gpointer data)
 {
@@ -1425,13 +1425,13 @@ find_label_child (GtkContainer *container,
                       child = NULL;
                       break;
                     }
-                  child = GTK_WIDGET (tmp_list->data);
+                  child = widget;
                 }
               else
                 {
                   if (*index == 0)
                     {
-	              child = GTK_WIDGET (tmp_list->data);
+	              child = widget;
                       break;
                     }
                   *index--;
@@ -1473,7 +1473,7 @@ get_label_from_button (GtkWidget *button,
   return child;
 }
 
-static void*
+static void
 count_labels (GtkContainer *container,
               gint         *n_labels)
 {
