@@ -229,7 +229,8 @@ gail_notebook_real_notify_gtk (GObject           *obj,
   if (strcmp (pspec->name, "page") == 0)
     {
       gint page_num, old_page_num;
-      gint focus_page_num, old_focus_page_num;
+      gint focus_page_num = 0;
+      gint old_focus_page_num;
       GailNotebook *gail_notebook = GAIL_NOTEBOOK (atk_obj);
       GtkNotebook *gtk_notebook = GTK_NOTEBOOK (widget);
      
@@ -240,8 +241,11 @@ gail_notebook_real_notify_gtk (GObject           *obj,
       page_num = gtk_notebook_get_current_page (gtk_notebook);
       gail_notebook->selected_page = page_num;
       old_focus_page_num = gail_notebook->focus_tab_page;
-      focus_page_num = g_list_index (gtk_notebook->children, gtk_notebook->focus_tab->data);
-      gail_notebook->focus_tab_page = focus_page_num;
+      if (gtk_notebook->focus_tab && gtk_notebook->focus_tab->data)
+        {
+          focus_page_num = g_list_index (gtk_notebook->children, gtk_notebook->focus_tab->data);
+          gail_notebook->focus_tab_page = focus_page_num;
+        }
     
       if (page_num != old_page_num)
         {
