@@ -48,6 +48,7 @@ static gboolean              gail_window_real_focus_gtk (GtkWidget     *widget,
                                                          GdkEventFocus *event);
 
 static AtkStateSet*          gail_window_ref_state_set  (AtkObject     *accessible);
+static AtkRelationSet*       gail_window_ref_relation_set  (AtkObject     *accessible);
 static void                  gail_window_real_notify_gtk (GObject      *obj,
                                                           GParamSpec   *pspec);
 static gint                  gail_window_get_mdi_zorder (AtkComponent  *component);
@@ -127,6 +128,7 @@ gail_window_class_init (GailWindowClass *klass)
   class->get_name = gail_window_get_name;
   class->get_parent = gail_window_get_parent;
   class->get_index_in_parent = gail_window_get_index_in_parent;
+  class->ref_relation_set = gail_window_ref_relation_set;
   class->ref_state_set = gail_window_ref_state_set;
   class->initialize = gail_window_real_initialize;
 
@@ -230,10 +232,13 @@ gail_window_real_initialize (AtkObject *obj,
                              gpointer  data)
 {
   GtkWidget *widget;
+  GailWindow *window;
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  GAIL_WINDOW (obj)->name_change_handler = 0;
+  window = GAIL_WINDOW (obj);
+  window->name_change_handler = 0;
+  window->previous_name = g_strdup (gtk_window_get_title (GTK_WINDOW (data)));
   widget = GTK_WIDGET (data);
 
   g_signal_connect (data,
@@ -253,7 +258,7 @@ gail_window_real_initialize (AtkObject *obj,
     obj->role = ATK_ROLE_ALERT;
   else if (GTK_IS_DIALOG (widget))
     obj->role = ATK_ROLE_DIALOG;
-  else if (GTK_IS_WINDOW (widget))
+  else
     {
       const gchar *name;
 
@@ -262,13 +267,11 @@ gail_window_real_initialize (AtkObject *obj,
         obj->role = ATK_ROLE_TOOL_TIP;
       else if (GTK_IS_PLUG (widget))
         obj->role = ATK_ROLE_PANEL;
+      else if (GTK_WINDOW (widget)->type == GTK_WINDOW_POPUP)
+        obj->role = ATK_ROLE_WINDOW;
       else
         obj->role = ATK_ROLE_FRAME;
     }
-  else if (GTK_IS_HANDLE_BOX (widget))
-    obj->role = ATK_ROLE_UNKNOWN;
-  else
-    obj->role = ATK_ROLE_INVALID;
 }
 
 static void
@@ -281,6 +284,11 @@ gail_window_finalize (GObject *object)
       g_source_remove (window->name_change_handler);
       window->name_change_handler = 0;
     }
+  if (window->previous_name)
+    {
+      g_free (window->previous_name);
+      window->previous_name = NULL;
+    }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -288,10 +296,10 @@ gail_window_finalize (GObject *object)
 static G_CONST_RETURN gchar*
 gail_window_get_name (AtkObject *accessible)
 {
-  if (accessible->name)
-    return accessible->name;
+  G_CONST_RETURN gchar* name;
 
-  else
+  name = ATK_OBJECT_CLASS (parent_class)->get_name (accessible);
+  if (name == NULL)
     {
       /*
        * Get the window title if it exists
@@ -309,25 +317,20 @@ gail_window_get_name (AtkObject *accessible)
       if (GTK_IS_WINDOW (widget))
         {
           GtkWindow *window = GTK_WINDOW (widget);
-          G_CONST_RETURN gchar* name;
  
           name = gtk_window_get_title (window);
-          if (name)
-            return name;
-
-          if (accessible->role == ATK_ROLE_TOOL_TIP)
+          if (name == NULL &&
+              accessible->role == ATK_ROLE_TOOL_TIP)
             {
               GtkWidget *child;
 
               child = gtk_bin_get_child (GTK_BIN (window));
               g_return_val_if_fail (GTK_IS_LABEL (child), NULL);
               name = gtk_label_get_text (GTK_LABEL (child));
-              return name;
             }
         }
-                    
-      return ATK_OBJECT_CLASS (parent_class)->get_name (accessible);
     }
+  return name;
 }
 
 static AtkObject*
@@ -381,6 +384,48 @@ gail_window_real_focus_gtk (GtkWidget     *widget,
   return FALSE;
 }
 
+static AtkRelationSet*
+gail_window_ref_relation_set (AtkObject *obj)
+{
+  GtkWidget *widget;
+  AtkRelationSet *relation_set;
+  AtkObject *array[1];
+  AtkRelation* relation;
+  GtkWidget *current_widget;
+  gboolean ret;
+
+  g_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
+
+  widget = GTK_ACCESSIBLE (obj)->widget;
+  if (widget == NULL)
+    /*
+     * State is defunct
+     */
+    return NULL;
+
+  relation_set = ATK_OBJECT_CLASS (parent_class)->ref_relation_set (obj);
+
+  if (atk_object_get_role (obj) == ATK_ROLE_TOOL_TIP)
+    {
+      relation = atk_relation_set_get_relation_by_type (relation_set, ATK_RELATION_POPUP_FOR);
+
+      if (relation)
+        {
+          atk_relation_set_remove (relation_set, relation);
+        }
+      ret = gtk_tooltips_get_info_from_tip_window (GTK_WINDOW (widget), NULL, &current_widget);
+      if (ret)
+        {
+          array [0] = gtk_widget_get_accessible (current_widget);
+
+          relation = atk_relation_new (array, 1, ATK_RELATION_POPUP_FOR);
+          atk_relation_set_add (relation_set, relation);
+          g_object_unref (relation);
+        }
+    }
+  return relation_set;
+}
+
 static AtkStateSet*
 gail_window_ref_state_set (AtkObject *accessible)
 {
@@ -445,11 +490,29 @@ gail_window_real_notify_gtk (GObject		*obj,
   GtkWidget *widget = GTK_WIDGET (obj);
   AtkObject* atk_obj = gtk_widget_get_accessible (widget);
   GailWindow *window = GAIL_WINDOW (atk_obj);
+  const gchar *name;
+  gboolean name_changed = FALSE;
 
   if (strcmp (pspec->name, "title") == 0)
     {
-      if (window->name_change_handler == 0)
-        window->name_change_handler = g_idle_add (idle_notify_name_change, atk_obj);    
+      name = gtk_window_get_title (GTK_WINDOW (widget));
+      if (name)
+        {
+         if (window->previous_name == NULL ||
+             strcmp (name, window->previous_name) != 0)
+           name_changed = TRUE;
+        }
+      else if (window->previous_name != NULL)
+        name_changed = TRUE;
+
+      if (name_changed)
+        {
+          g_free (window->previous_name);
+          window->previous_name = g_strdup (name);
+       
+          if (window->name_change_handler == 0)
+            window->name_change_handler = g_idle_add (idle_notify_name_change, atk_obj);    
+        }
     }
   else
     GAIL_WIDGET_CLASS (parent_class)->notify_gtk (obj, pspec);
@@ -587,7 +650,7 @@ get_window_desktop (Window window)
   int             format;
   gulong          nitems;
   gulong          bytes_after;
-  gulong         *cardinals;
+  guchar         *cardinals;
   int             error;
   int             result;
   int             desktop;
@@ -601,7 +664,7 @@ get_window_desktop (Window window)
                                0, G_MAXLONG,
                                False, XA_CARDINAL,
                                &ret_type, &format, &nitems,
-                               &bytes_after, (guchar **) &cardinals);
+                               &bytes_after, &cardinals);
   error = gdk_error_trap_pop();
   /* nitems < 1 will occur if the property is not set */
   if (error != Success || result != Success || nitems < 1)
@@ -638,7 +701,7 @@ get_stacked_windows (GailScreenInfo *info)
   int     format;
   gulong  nitems;
   gulong  bytes_after;
-  Window *data;
+  guchar *data;
   int     error;
   int     result;
   int     i;
@@ -657,7 +720,7 @@ get_stacked_windows (GailScreenInfo *info)
                                _net_client_list_stacking,
                                0, G_MAXLONG,
                                False, XA_WINDOW, &ret_type, &format, &nitems,
-                               &bytes_after, (guchar **)&data);
+                               &bytes_after, &data);
   error = gdk_error_trap_pop ();
   /* nitems < 1 will occur if the property is not set */
   if (error != Success || result != Success || nitems < 1)
@@ -696,7 +759,7 @@ get_stacked_windows (GailScreenInfo *info)
         }
     }
   free_screen_info (info);
-  info->stacked_windows = data;
+  info->stacked_windows = (Window*) data;
   info->stacked_windows_len = nitems;
   info->desktop = desktops;
   info->desktop_changed = desktops_changed;
